import React, { Fragment, useEffect, useRef, useState } from 'react';
import { renderSidebarPart, renderActivitybarPar, renderEditorPart, renderPanelPart, renderStatusBarPart, registerCustomView } from 'vscode/service-override/views';
import { createPortal } from 'react-dom';
import { initializePromise } from '@codingame/monaco-editor-wrapper';
import { DisposableStore } from 'vscode/monaco';
function createPart(location, renderPart) {
    const element = document.createElement('div');
    element.style.flex = '1';
    element.style.minWidth = '0';
    initializePromise.then(() => {
        renderPart(element);
    }, console.error);
    return function VscodePartRenderer({ views = [], className, style }) {
        const ref = useRef(null);
        useEffect(() => {
            ref.current?.append(element);
        }, []);
        const [portalComponents, setPortalComponents] = useState({});
        useEffect(() => {
            if (location == null) {
                return;
            }
            const disposableStore = new DisposableStore();
            for (const view of views) {
                disposableStore.add(registerCustomView({
                    id: view.id,
                    name: view.name,
                    order: view.order,
                    renderBody: function (container) {
                        setPortalComponents(portalComponents => ({
                            ...portalComponents,
                            [view.id]: container
                        }));
                        return {
                            dispose() {
                                setPortalComponents(portalComponents => {
                                    const { [view.id]: _, ...remaining } = portalComponents;
                                    return remaining;
                                });
                            }
                        };
                    },
                    location,
                    icon: view.icon,
                    actions: view.actions?.map(action => {
                        return {
                            ...action,
                            render: action.node != null
                                ? (container) => {
                                    setPortalComponents(portalComponents => ({
                                        ...portalComponents,
                                        [action.id]: container
                                    }));
                                    return {
                                        dispose() {
                                            setPortalComponents(portalComponents => {
                                                const { [action.id]: _, ...remaining } = portalComponents;
                                                return remaining;
                                            });
                                        }
                                    };
                                }
                                : undefined
                        };
                    })
                }));
            }
            return () => {
                disposableStore.dispose();
            };
        }, [views]);
        return (React.createElement(React.Fragment, null,
            views.filter(view => view.id in portalComponents).map((view) => {
                return (React.createElement(Fragment, { key: view.id },
                    createPortal(view.node, portalComponents[view.id], view.id),
                    view.actions?.filter(action => action.id in portalComponents).map(action => (React.createElement(Fragment, { key: action.id }, createPortal(action.node, portalComponents[action.id], action.id))))));
            }),
            React.createElement("div", { ref: ref, className: className, style: {
                    display: 'flex',
                    alignItems: 'stretch',
                    justifyContent: 'stretch',
                    ...style
                } })));
    };
}
export const SidebarPart = createPart(0 /* ViewContainerLocation.Sidebar */, renderSidebarPart);
export const ActivitybarPart = createPart(null, renderActivitybarPar);
export const EditorPart = createPart(null, renderEditorPart);
export const StatusBarPart = createPart(null, renderStatusBarPart);
export const PanelPart = createPart(1 /* ViewContainerLocation.Panel */, renderPanelPart);
//# sourceMappingURL=VscodePart.js.map